==> frgal-3.10.1-py3-none-any.whl: frgal/tests/test_decoder.py <==
"""Tests for decoder module."""
from datetime import datetime
from typing import Dict, List, cast
from unittest import TestCase

import pytz
from google.protobuf.empty_pb2 import Empty
from google.protobuf.struct_pb2 import Struct
from google.protobuf.timestamp_pb2 import Timestamp
from testfixtures import ShouldWarn

from frgal.decoder import GrpcDecoder
from frgal.exceptions import DecoderError, UnknownError

from .proto.example_pb2 import (ComplexMessage, ComplexNestedMessage, CompositeDictMessage, CompositeListMessage,
                                DictMessage, ListMessage, NestedMessage, Reply, SimpleMessage)


class TestGrpcDecoder(TestCase):
    """Test GrpcDecoder."""

    def setUp(self):
        self.decoder = GrpcDecoder()

    # Test various message structures
    def test_decode_empty(self):
        self.assertIsNone(self.decoder.decode(Empty()))

    def test_decode_empty_message(self):
        self.assertEqual(self.decoder.decode(SimpleMessage()), None)

    def test_decode_onefield_message(self):
        msg = SimpleMessage()
        msg.value = 'Arnold Rimmer'

        self.assertEqual(self.decoder.decode(msg), 'Arnold Rimmer')

    def test_decode_complex_message(self):
        msg = ComplexMessage()
        msg.value = 'Arnold Rimmer'
        msg.count = 42

        self.assertEqual(self.decoder.decode(msg), {'value': 'Arnold Rimmer', 'count': 42})

    def test_decode_nested_empty(self):
        msg = NestedMessage()

        self.assertEqual(self.decoder.decode(msg), None)

    def test_decode_nested(self):
        msg = NestedMessage()
        msg.value.value = 'Arnold'

        self.assertEqual(self.decoder.decode(msg), 'Arnold')

    def test_decode_with_primitive_type_default_values(self):
        self.assertEqual(self.decoder.decode(ComplexMessage()), {'value': None, 'count': 0})

    # Test various value types
    def test_decode_list_empty(self):
        msg = ListMessage()
        self.assertEqual(self.decoder.decode(msg), [])

    def test_decode_list(self):
        msg = ListMessage()
        msg.values.append('Arnold')
        msg.values.append('Lister')

        self.assertEqual(self.decoder.decode(msg), ['Arnold', 'Lister'])

    def test_decode_composite_list_empty(self):
        msg = CompositeListMessage()
        self.assertEqual(self.decoder.decode(msg), [])

    def test_decode_composite_list(self):
        msg = CompositeListMessage()
        arnold = SimpleMessage()
        arnold.value = 'Arnold'
        lister = SimpleMessage()
        lister.value = 'Lister'
        msg.values.MergeFrom([arnold, lister])

        self.assertEqual(self.decoder.decode(msg), ['Arnold', 'Lister'])

    def test_decode_dict_empty(self):
        msg = DictMessage()
        self.assertEqual(self.decoder.decode(msg), {})

    def test_decode_dict(self):
        msg = DictMessage()
        msg.values['arnold'] = 'Arnold'
        msg.values['lister'] = 'Lister'

        self.assertEqual(self.decoder.decode(msg), {'arnold': 'Arnold', 'lister': 'Lister'})

    def test_decode_composite_dict_empty(self):
        msg = CompositeDictMessage()
        self.assertEqual(self.decoder.decode(msg), {})

    def test_decode_composite_dict(self):
        msg = CompositeDictMessage()
        msg.values['arnold'].value = 'Arnold'
        msg.values['lister'].value = 'Lister'

        self.assertEqual(self.decoder.decode(msg), {'arnold': 'Arnold', 'lister': 'Lister'})

    def test_decode_unknown_type(self):
        with self.assertRaises(DecoderError):
            self.decoder.decode(object()),

    def test_decode_timestamp(self):
        self.assertIsNone(self.decoder.decode(Timestamp()))

        self.assertEqual(
            self.decoder.decode(Timestamp(nanos=1)),
            datetime(1970, 1, 1, 0, 0, 0, 0, tzinfo=pytz.utc)
        )

        self.assertEqual(
            self.decoder.decode(Timestamp(seconds=1)),
            datetime(1970, 1, 1, 0, 0, 1, 0, tzinfo=pytz.utc)
        )

        self.assertEqual(
            self.decoder.decode(Timestamp(seconds=123456789, nanos=12345678)),
            datetime(1973, 11, 29, 21, 33, 9, 12345, tzinfo=pytz.utc)
        )

    def test_decode_oneof_empty(self):
        # Test decoding of empty oneof fields
        reply = Reply()

        self.assertEqual(self.decoder.decode(reply), {})

    def test_decode_oneof(self):
        # Test decoding of oneof fields
        reply = Reply()
        reply.data.result = 'Gazpacho!'

        self.assertEqual(self.decoder.decode(reply), {'data': 'Gazpacho!'})

    def test_decode_struct(self):
        data = [
            {},
            {'null': None},
            {'true': True},
            {'false': False},
            {'zero': 0},
            {'integer': 42},
            {'empty_string': ''},
            {'string': 'string'},
        ]  # type: List[Dict]
        data += [{'nested_list': list(content.values())} for content in data]
        data += [{'nested_dict': content} for content in data]
        for content in data:
            with self.subTest(content=content):
                struct = Struct()
                struct.update(content)
                self.assertEqual(self.decoder.decode(struct), content)

    # Test exceptions
    def test_set_exception_decoder_missing_args(self):
        with self.assertRaises(TypeError):
            self.decoder.set_exception_decoder()
        with self.assertRaises(TypeError):
            self.decoder.set_exception_decoder(Reply.Exception.TeaPotError)
        with self.assertRaises(TypeError):
            self.decoder.set_exception_decoder(exception=Exception)

    def test_set_exception_decoder_name_arg(self):
        with ShouldWarn(DeprecationWarning('Argument name of method set_exception_decoder is deprecated.')):
            self.decoder.set_exception_decoder(name='exception', exception=Exception)

    def test_decode_exception_by_name(self):
        # Add exception decoder
        class TeaPotError(Exception):
            pass

        with ShouldWarn(DeprecationWarning('Decoding exceptions by name is deprecated. Use decoding by type instead.')):
            self.decoder.set_exception_decoder('tea_pot_error', TeaPotError)

        reply = Reply()
        reply.exception.tea_pot_error.CopyFrom(Reply.Exception.TeaPotError())
        self.assertIsInstance(self.decoder.decode_exception(reply.exception), TeaPotError)

    def test_decode_exception_by_type(self):
        # Add exception decoder
        class TeaPotError(Exception):
            def __init__(self, temperature, plugged_in):
                self.temperature = temperature
                self.plugged_in = plugged_in

        self.decoder.set_exception_decoder(Reply.Exception.TeaPotError, TeaPotError)

        reply = Reply()
        reply.exception.tea_pot_error.CopyFrom(Reply.Exception.TeaPotError(temperature=42, plugged_in=True))
        decoded = cast(TeaPotError, self.decoder.decode_exception(reply.exception))
        self.assertIsInstance(decoded, TeaPotError)
        self.assertEqual(decoded.temperature, 42)
        self.assertTrue(decoded.plugged_in)

    def test_decode_exception_empty(self):
        error = Reply.Exception()
        self.assertIsInstance(self.decoder.decode_exception(error), UnknownError)

    def test_decode_exception_unknown(self):
        error = Reply.Exception()
        error.something_went_wrong.CopyFrom(Reply.Exception.SomethingWentWrong())
        self.assertIsInstance(self.decoder.decode_exception(error), UnknownError)

    # Test shallow_fields
    def test_shallow_fields_full(self):
        self.decoder.set_shallow_fields(ComplexNestedMessage, ['value1', 'value2'])
        message = ComplexNestedMessage()
        message.value1.value = 'abcdef'
        message.value2.count = 42
        self.assertEqual(self.decoder.decode(message), {
            'value1': {'count': 0, 'value': 'abcdef'},
            'value2': {'count': 42, 'value': None},
        })

    def test_shallow_fields_empty(self):
        self.decoder.set_shallow_fields(ComplexNestedMessage, ['value1', 'value2'])
        message = ComplexNestedMessage()
        self.assertEqual(self.decoder.decode(message), {
            'value1': None,
            'value2': None,
        })

    def test_not_shallow_fields(self):
        self.decoder.set_shallow_fields(ComplexNestedMessage, [])
        message = ComplexNestedMessage()
        self.assertEqual(self.decoder.decode(message), {
            'value1': {'count': 0, 'value': None},
            'value2': {'count': 0, 'value': None},
        })

==> frgal-3.12.4-py3-none-any.whl: frgal/tests/test_decoder.py <==
"""Tests for decoder module."""
from datetime import datetime
from typing import Dict, List, cast
from unittest import TestCase

import pytz
from google.protobuf.empty_pb2 import Empty
from google.protobuf.struct_pb2 import Struct
from google.protobuf.timestamp_pb2 import Timestamp
from testfixtures import ShouldWarn

from frgal.decoder import GrpcDecoder
from frgal.exceptions import DecoderError, UnknownError

from .proto.example_pb2 import (ComplexMessage, ComplexNestedMessage, ComplexOneofMessage,  # type: ignore[attr-defined]
                                ComplexOptionalMessage, CompositeDictMessage, CompositeListMessage, DictMessage,
                                ListMessage, NestedMessage, Reply, SimpleMessage)

DECODE_UNSET_DEPRECATED = '%s: Decoding of not set message fields will be disabled in the future.'
SHALLOW_FIELDS_DEPRECATED = ('Shallow fields are deprecated by `decode_unset_messages`. '
                             'This method will be removed in the future.')
DECODE_STRING_NONE_DEPRECATED = (
    'Decoding non-optional empty strings to None is deprecated and will be removed in the next major release. '
    'Use `decode_empty_string_none=False` to be future-compatible.'
)


class GrpcDecoderDecodeNotSetTest(TestCase):
    """Test GrpcDecoder."""

    def setUp(self):
        self.decoder = GrpcDecoder()

    def test_init(self):
        self.assertTrue(GrpcDecoder().decode_unset_messages)
        self.assertTrue(GrpcDecoder(decode_unset_messages=True).decode_unset_messages)
        self.assertFalse(GrpcDecoder(decode_unset_messages=False).decode_unset_messages)

    # Test various message structures
    def test_decode_empty(self):
        self.assertIsNone(self.decoder.decode(Empty()))

    def test_decode_empty_message(self):
        self.assertEqual(self.decoder.decode(SimpleMessage()), None)

    def test_decode_onefield_message(self):
        msg = SimpleMessage()
        msg.value = 'Arnold Rimmer'

        self.assertEqual(self.decoder.decode(msg), 'Arnold Rimmer')

    def test_decode_complex_message(self):
        msg = ComplexMessage()
        msg.value = 'Arnold Rimmer'
        msg.count = 42

        self.assertEqual(self.decoder.decode(msg), {'value': 'Arnold Rimmer', 'count': 42})

    def test_decode_str_none(self):
        data = (
            (SimpleMessage(value=''), None),
            (ComplexMessage(), {'value': None, 'count': 0}),
        )
        for message, output in data:
            with self.subTest(message=message, output=output):
                with ShouldWarn(DeprecationWarning(DECODE_STRING_NONE_DEPRECATED)):
                    self.assertEqual(self.decoder.decode(message), output)

    def test_decode_str_empty(self):
        data = (
            (SimpleMessage(value=''), ''),
            (ComplexMessage(), {'value': '', 'count': 0}),
        )
        decoder = TestGrpcDecoder(decode_empty_string_none=False)
        for message, output in data:
            with self.subTest(message=message, output=output):
                self.assertEqual(decoder.decode(message), output)

    def test_decode_nested_empty(self):
        msg = NestedMessage()

        with ShouldWarn(
            DeprecationWarning(DECODE_UNSET_DEPRECATED % ('NestedMessage.value',)),
            DeprecationWarning(DECODE_STRING_NONE_DEPRECATED),
        ):
            self.assertEqual(self.decoder.decode(msg), None)

    def test_decode_nested(self):
        msg = NestedMessage()
        msg.value.value = 'Arnold'

        self.assertEqual(self.decoder.decode(msg), 'Arnold')

    def test_decode_with_primitive_type_default_values(self):
        self.assertEqual(self.decoder.decode(ComplexMessage()), {'value': None, 'count': 0})

    # Test various value types
    def test_decode_list_empty(self):
        msg = ListMessage()
        self.assertEqual(self.decoder.decode(msg), [])

    def test_decode_list(self):
        msg = ListMessage()
        msg.values.append('Arnold')
        msg.values.append('Lister')

        self.assertEqual(self.decoder.decode(msg), ['Arnold', 'Lister'])

    def test_decode_composite_list_empty(self):
        msg = CompositeListMessage()
        self.assertEqual(self.decoder.decode(msg), [])

    def test_decode_composite_list(self):
        msg = CompositeListMessage()
        arnold = SimpleMessage()
        arnold.value = 'Arnold'
        lister = SimpleMessage()
        lister.value = 'Lister'
        msg.values.MergeFrom([arnold, lister])

        self.assertEqual(self.decoder.decode(msg), ['Arnold', 'Lister'])

    def test_decode_dict_empty(self):
        msg = DictMessage()
        self.assertEqual(self.decoder.decode(msg), {})

    def test_decode_dict(self):
        msg = DictMessage()
        msg.values['arnold'] = 'Arnold'
        msg.values['lister'] = 'Lister'

        self.assertEqual(self.decoder.decode(msg), {'arnold': 'Arnold', 'lister': 'Lister'})

    def test_decode_composite_dict_empty(self):
        msg = CompositeDictMessage()
        self.assertEqual(self.decoder.decode(msg), {})

    def test_decode_composite_dict(self):
        msg = CompositeDictMessage()
        msg.values['arnold'].value = 'Arnold'
        msg.values['lister'].value = 'Lister'

        self.assertEqual(self.decoder.decode(msg), {'arnold': 'Arnold', 'lister': 'Lister'})

    def test_decode_unknown_type(self):
        with self.assertRaises(DecoderError):
            self.decoder.decode(object()),

    def test_decode_timestamp(self):
        self.assertIsNone(self.decoder.decode(Timestamp()))

        self.assertEqual(
            self.decoder.decode(Timestamp(nanos=1)),
            datetime(1970, 1, 1, 0, 0, 0, 0, tzinfo=pytz.utc)
        )

        self.assertEqual(
            self.decoder.decode(Timestamp(seconds=1)),
            datetime(1970, 1, 1, 0, 0, 1, 0, tzinfo=pytz.utc)
        )

        self.assertEqual(
            self.decoder.decode(Timestamp(seconds=123456789, nanos=12345678)),
            datetime(1973, 11, 29, 21, 33, 9, 12345, tzinfo=pytz.utc)
        )

    def test_decode_oneof_empty(self):
        message = ComplexOneofMessage()
        self.assertEqual(self.decoder.decode(message), {})

    def test_decode_oneof_full(self):
        message = ComplexOneofMessage()
        message.value = 0
        message.value2 = 2
        self.assertEqual(self.decoder.decode(message), {'value': 0, 'value2': 2})

    def test_decode_oneof(self):
        # Test decoding of oneof fields
        reply = Reply()
        reply.data.result = 'Gazpacho!'

        self.assertEqual(self.decoder.decode(reply), {'data': 'Gazpacho!'})

    def test_decode_struct(self):
        data = [
            {},
            {'null': None},
            {'true': True},
            {'false': False},
            {'zero': 0},
            {'integer': 42},
            {'empty_string': ''},
            {'string': 'string'},
        ]  # type: List[Dict]
        data += [{'nested_list': list(content.values())} for content in data]
        data += [{'nested_dict': content} for content in data]
        for content in data:
            with self.subTest(content=content):
                struct = Struct()
                struct.update(content)
                self.assertEqual(self.decoder.decode(struct), content)

    # Test exceptions
    def test_set_exception_decoder_missing_args(self):
        with self.assertRaises(TypeError):
            self.decoder.set_exception_decoder()
        with self.assertRaises(TypeError):
            self.decoder.set_exception_decoder(Reply.Exception.TeaPotError)
        with self.assertRaises(TypeError):
            self.decoder.set_exception_decoder(exception=Exception)

    def test_set_exception_decoder_name_arg(self):
        with ShouldWarn(DeprecationWarning('Argument name of method set_exception_decoder is deprecated.')):
            self.decoder.set_exception_decoder(name='exception', exception=Exception)

    def test_decode_exception_by_name(self):
        # Add exception decoder
        class TeaPotError(Exception):
            pass

        with ShouldWarn(DeprecationWarning('Decoding exceptions by name is deprecated. Use decoding by type instead.')):
            self.decoder.set_exception_decoder('tea_pot_error', TeaPotError)

        reply = Reply()
        reply.exception.tea_pot_error.CopyFrom(Reply.Exception.TeaPotError())
        self.assertIsInstance(self.decoder.decode_exception(reply.exception), TeaPotError)

    def test_decode_exception_by_type(self):
        # Add exception decoder
        class TeaPotError(Exception):
            def __init__(self, temperature, plugged_in):
                self.temperature = temperature
                self.plugged_in = plugged_in

        self.decoder.set_exception_decoder(Reply.Exception.TeaPotError, TeaPotError)

        reply = Reply()
        reply.exception.tea_pot_error.CopyFrom(Reply.Exception.TeaPotError(temperature=42, plugged_in=True))
        decoded = cast(TeaPotError, self.decoder.decode_exception(reply.exception))
        self.assertIsInstance(decoded, TeaPotError)
        self.assertEqual(decoded.temperature, 42)
        self.assertTrue(decoded.plugged_in)

    def test_decode_exception_one_field(self):
        # Add exception decoder
        class MagicError(Exception):
            def __init__(self, thaum):
                self.thaum = thaum

        self.decoder.set_exception_decoder(Reply.Exception.MagicError, MagicError)

        error = Reply.Exception()
        error.magic_error.CopyFrom(Reply.Exception.MagicError(thaum=42))
        decoded = self.decoder.decode_exception(error)
        self.assertIsInstance(decoded, MagicError)
        self.assertEqual(decoded.thaum, 42)

    def test_decode_exception_empty(self):
        error = Reply.Exception()
        self.assertIsInstance(self.decoder.decode_exception(error), UnknownError)

    def test_decode_exception_unknown(self):
        error = Reply.Exception()
        error.something_went_wrong.CopyFrom(Reply.Exception.SomethingWentWrong())
        self.assertIsInstance(self.decoder.decode_exception(error), UnknownError)

    # Test shallow_fields
    def test_shallow_fields_full(self):
        with ShouldWarn(DeprecationWarning(SHALLOW_FIELDS_DEPRECATED)):
            self.decoder.set_shallow_fields(ComplexNestedMessage, ['value1', 'value2'])
        message = ComplexNestedMessage()
        message.value1.value = 'abcdef'
        message.value2.count = 42
        self.assertEqual(self.decoder.decode(message), {
            'value1': {'count': 0, 'value': 'abcdef'},
            'value2': {'count': 42, 'value': None},
        })

    def test_shallow_fields_empty(self):
        with ShouldWarn(DeprecationWarning(SHALLOW_FIELDS_DEPRECATED)):
            self.decoder.set_shallow_fields(ComplexNestedMessage, ['value1', 'value2'])
        message = ComplexNestedMessage()
        self.assertEqual(self.decoder.decode(message), {
            'value1': None,
            'value2': None,
        })

    def test_not_shallow_fields(self):
        with ShouldWarn(DeprecationWarning(SHALLOW_FIELDS_DEPRECATED)):
            self.decoder.set_shallow_fields(ComplexNestedMessage, [])
        message = ComplexNestedMessage()
        with ShouldWarn(
            DeprecationWarning(DECODE_UNSET_DEPRECATED % ('ComplexNestedMessage.value1',)),
            DeprecationWarning(DECODE_STRING_NONE_DEPRECATED),
            DeprecationWarning(DECODE_UNSET_DEPRECATED % ('ComplexNestedMessage.value2',)),
            DeprecationWarning(DECODE_STRING_NONE_DEPRECATED),
        ):
            self.assertEqual(self.decoder.decode(message), {
                'value1': {'count': 0, 'value': None},
                'value2': {'count': 0, 'value': None},
            })

    def test_optional_fields_full(self):
        message = ComplexOptionalMessage()
        message.value = 'kryten'
        message.number = 42
        self.assertEqual(self.decoder.decode(message), {
            'value': 'kryten',
            'number': 42,
        })

    def test_optional_fields_empty(self):
        message = ComplexOptionalMessage()
        self.assertEqual(self.decoder.decode(message), {
            'value': None,
            'number': None,
        })


class TestGrpcDecoder(GrpcDecoder):
    decode_unset_messages = False


class GrpcDecoderTest(GrpcDecoderDecodeNotSetTest):
    """Test GrpcDecoder with `decode_unset_messages` disabled."""

    def setUp(self):
        self.decoder = TestGrpcDecoder()

    def test_decode_nested_empty(self):
        msg = NestedMessage()
        self.assertEqual(self.decoder.decode(msg), None)

    def test_shallow_fields_empty(self):
        with ShouldWarn(DeprecationWarning(SHALLOW_FIELDS_DEPRECATED)):
            self.decoder.set_shallow_fields(ComplexNestedMessage, ['value1', 'value2'])
        message = ComplexNestedMessage()
        self.assertEqual(self.decoder.decode(message), {
            'value1': None,
            'value2': None,
        })

    def test_not_shallow_fields(self):
        with ShouldWarn(DeprecationWarning(SHALLOW_FIELDS_DEPRECATED)):
            self.decoder.set_shallow_fields(ComplexNestedMessage, [])
        message = ComplexNestedMessage()
        self.assertEqual(self.decoder.decode(message), {
            'value1': None,
            'value2': None,
        })
